#include "sortiva.hpp"

#include <cstdlib>
#include <stdio.h>


Sortiva::Sortiva() : wnd(1280, 720, "Sortiva", rl::FLAG_WINDOW_RESIZABLE | rl::FLAG_WINDOW_TRANSPARENT)
{
	if (!wnd.IsWindowReady())
	{
		printf("Window could not be initialized\n");
		exit(-1);
	}
	rl::SetWindowState(rl::FLAG_WINDOW_RESIZABLE | rl::FLAG_WINDOW_TRANSPARENT);

	m_Width = rl::GetScreenWidth();
	m_Height = rl::GetScreenHeight();

	rl::SetTargetFPS(60);
}

Sortiva::~Sortiva()
{
	rl::CloseWindow();
}

void Sortiva::run()
{
	setup();
	while (m_Running)
	{
		if (rl::WindowShouldClose()) m_Running = false;
		if (rl::IsWindowResized())
		{
			m_Width = rl::GetScreenWidth();
			m_Height = rl::GetScreenHeight();
		}

		rl::ClearBackground({ 25, 25, 25, 255 });
		rl::BeginDrawing();

		draw();

		rl::EndDrawing();
	}
}

constexpr rl::Color sorter_colors[] = {
	{ 253, 249, 0, 255 },	// Yellow
	{ 255, 203, 0, 255 },	// Gold
	{ 255, 161, 0, 255 },	// Orange
	{ 255, 109, 194, 255 },	// Pink
	{ 230, 41, 55, 255 },	// Red
	{ 190, 33, 55, 255 },	// Maroon
	{ 0, 228, 48, 255 },		// Green
	{ 0, 158, 47, 255 },		// Lime
	{ 0, 117, 44, 255 },		// Dark Green
	{ 102, 191, 255, 255 },	// Sky Blue
	{ 0, 121, 241, 255 },	// Blue
	{ 0, 82, 172, 255 },		// Dark Blue
	{ 255, 0, 255, 255 },	// Magenta
	{ 200, 122, 255, 255 },	// Purple
	{ 135, 60, 190, 255 },	// Violet
	{ 112, 31, 126, 255 },	// Dark Purple
	{ 211, 176, 131, 255 },	// Beige
	{ 127, 106, 79, 255 },	// Brown
	{ 76, 63, 47, 255 },		// Dark Brown
};

constexpr int sorter_colors_size = sizeof(sorter_colors) / sizeof(rl::Color);

constexpr rl::Color sorter_block_colors[] = {
	{ 200, 200, 200, 255 },	// Light Gray
	{ 80, 80, 80, 255 },		// Dark Gray
};

constexpr int sorter_block_colors_size = sizeof(sorter_block_colors) / sizeof(rl::Color);

void Sortiva::draw()
{
	float margin = static_cast<float>(m_Width) / m_Height * 10;
	rl::DrawRectangle(1 + margin, 1 + margin, m_Width - 2 * margin, m_Height - 2 * margin, { 60,60,60,255 });

	margin = static_cast<float>(m_Width) / m_Height * 20;


	int steps = (m_Steps.size() / m_Values) + 2;

	float hoffset = (static_cast<float>(m_Height) - margin * 2) / static_cast<float>(m_Values);
	float woffset = (static_cast<float>(m_Width) - margin * 2) / static_cast<float>(steps);

	for (int i = 0; i < steps; ++i)
	{
		rl::DrawRectangle(
			margin + (woffset * static_cast<float>(i)),
			margin,
			woffset,
			m_Height - 2 * margin,
			sorter_block_colors[i % sorter_block_colors_size]
		);
	}


	steps = m_Steps.size() / m_Values;

	//  colors to values - ratio ; adds high variance to the color selection
	int colid = sorter_colors_size / m_Values;

	// font settings
	constexpr rl::Color textColor = RL_COLOR(BLACK);
	int fsize = margin;

	for (int i = 0; i < m_Values; ++i)
	{
		float hp = hoffset * (i + .5f);
		rl::Vector2 pos = { margin * 3.f, hp };
		rl::Color col = sorter_colors[(m_Steps[i] * colid) % sorter_colors_size];

		rl::DrawRing(pos, margin / 4, margin / 2, 20.f, 340.f, 10, col);

		rl::DrawLineEx(
			pos,
			{ woffset + margin, hp },
			margin / 4,
			col
		);

		rl::DrawText(
			rl::TextFormat("%d", m_Steps[i]),
			pos.x - 20 - fsize / 2,
			pos.y - fsize / 2,
			fsize,
			textColor
		);
	}

	for (int i = 0; i < m_Values; ++i)
	{
		float hp = hoffset * (i + .5f);
		for (int s = 0; s < steps; ++s) {
			rl::DrawLineEx(
				{ woffset * (s + 1) + margin, hp },
				{ woffset * (s + 2) + margin, hp },
				margin / 4,
				// calculating the 1d position; multiplies the colors/values ratio
				sorter_colors[(m_Steps[(m_Values * s) + i] * colid) % sorter_colors_size]
			);
		}
	}

	for (int i = 0; i < m_Values; ++i)
	{
		float hp = hoffset * (i + .5f);
		rl::Vector2 pos = { woffset * (steps + 2) - margin, hp };
		rl::Color col = sorter_colors[(m_Steps[m_Values * (steps - 1) + i] * colid) % sorter_colors_size];

		rl::DrawCircleV(pos, margin / 2, col);

		rl::DrawLineEx(
			{ woffset * (steps + 1) + margin, hp },
			pos,
			margin / 4,
			col
		);

		rl::DrawText(
			rl::TextFormat("%d", m_Steps[m_Values * (steps - 1) + i]),
			pos.x + margin / 2 + fsize / 2,
			pos.y - fsize / 2,
			fsize,
			textColor
		);
	}

}



void Sortiva::setup()
{
	m_Steps.clear();

	m_Steps = {
		2,1,3,
		1,2,3,
		3,1,2,
		1,2,3,
	};

}
